#pragma interface
#include "Screen.h"

struct FcbIO {
  typedef const char *Tag;
  virtual Tag first(int = 0) = 0;
  virtual Tag next(int = 1) = 0;
  Tag prev(int i = 1) { return next(-i); }
  virtual Tag locate(Tag tag = 0) = 0;
  virtual int taglen() const = 0;
  virtual Tag add(Tag = 0,int = 0) { return 0; }
  virtual Tag get(Tag tag) { return locate(tag); }
  virtual Tag update(Tag) { return 0; }
  virtual Tag del(Tag tag) { return 0; }
  virtual void blank() = 0;	// create blank record
  //virtual int isnull() const = 0;
  //virtual void setnull() = 0;
  virtual ~FcbIO() { };
};

struct Fcb: virtual Controller, virtual FcbIO {
  void lindup(short);			// create all lines
  Fcb();			// an Fcb of lcnt lines
  virtual ~Fcb();
  void getfocus();
  void key(int);
  void cmd(int);
  void disp();	// erase current data and redisplay
  void fwd();
  void rev();
  virtual void find();	// locate
  const char *rdlin();	// read record at cursor line
private:
  char *keyh;
  short tlen;
protected:
  short curline;
  short nlins;
  short last;
  bmsScreen **tbl;
  Tag *tagh;
  void puttbl(Tag ,short line = 0);
  virtual bmsScreen *liner(short) = 0;	// create the screen for each line
};

extern "C" long ldlong(const char *);
extern "C" void stlong(long,char *);

class ArrayIO: virtual public FcbIO {
  char tagbuf[4];
protected:
  int arridx;
public:
  ArrayIO() { arridx = 0; }
  int taglen() const { return sizeof tagbuf; }
  void tag(Tag p) { if (p) arridx = ldlong(p); }
  Tag tag() { if (arridx < 0) return 0; stlong(arridx,tagbuf); return tagbuf; }
  Tag first(int i) { arridx = i; return tag(); }
  Tag next(int i) { return first(arridx += i); }
  Tag locate(Tag p) { tag(p); return first(arridx); }
};
