// This may look like C code, but it is really -*- C++ -*-
/* 
Copyright (C) 1988 Free Software Foundation
    written by Doug Lea (dl@rocky.oswego.edu)

This file is part of the GNU C++ Library.  This library is free
software; you can redistribute it and/or modify it under the terms of
the GNU Library General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.  This library is distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef _SplayMap_h
#ifdef __GNUG__
//#pragma interface
#endif
#define _SplayMap_h 1

#include "Map.h"

template <class T,class C>
struct SplayNode {
  SplayNode*   lt;
  SplayNode*   rt;
  SplayNode*   par;
  T                item;
  C                cont;
                   SplayNode(T& h, C& c, 
                                     SplayNode* l=0, 
                                     SplayNode* r=0)
     :item(h), cont(c), lt(l), rt(r), par(0) {}
                   ~SplayNode() {}
};

template <class T,class C>
class SplayMap : public Map<T,C> {
  struct {
    SplayNode<T,C>*    lt;
    SplayNode<T,C>*    rt;
    SplayNode<T,C>*    par;
  } _dummy_null;
protected:
  SplayNode<T,C>*   root;

  SplayNode<T,C>*   leftmost();
  SplayNode<T,C>*   rightmost();
  SplayNode<T,C>*   pred(SplayNode<T,C>* t);
  SplayNode<T,C>*   succ(SplayNode<T,C>* t);
  void               _kill(SplayNode<T,C>* t);
  SplayNode<T,C>*   _copy(SplayNode<T,C>* t);

public:
               SplayMap(C& dflt);
               SplayMap(SplayMap& a);
               ~SplayMap() { _kill(root); }

  C&          operator [] (T& key);

  void          del(T& key);

  Pix           first() const;
  void          next(Pix& i);
  T&          key(Pix i);
  C&          contents(Pix i);

  Pix           seek(T& key);
  int           contains(T& key);

  void          clear(); 

  Pix           last();
  void          prev(Pix& i);

  int           OK();
};

template <class T,class C>
inline SplayMap<T,C>::SplayMap(C& dflt) :Map<T,C>(dflt) {
  root = 0;
}

template <class T,class C>
inline SplayMap<T,C>::SplayMap(SplayMap& b) :Map<T,C>(b.def) {
  count = b.count;
  root = _copy(b.root);
}

template <class T,class C>
inline Pix SplayMap<T,C>::first() const {
  return Pix(leftmost());
}

template <class T,class C>
inline Pix SplayMap<T,C>::last() {
  return Pix(rightmost());
}

template <class T,class C>
inline void SplayMap<T,C>::next(Pix& i) {
  if (i != 0) i = Pix(succ((SplayNode<T,C>*)i));
}

template <class T,class C>
inline void SplayMap<T,C>::prev(Pix& i) {
  if (i != 0) i = Pix(pred((SplayNode<T,C>*)i));
}

template <class T,class C>
inline T& SplayMap<T,C>::key (Pix i) {
  if (i == 0) error("null Pix");
  return ((SplayNode<T,C>*)i)->item;
}

template <class T,class C>
inline C& SplayMap<T,C>::contents (Pix i) {
  if (i == 0) error("null Pix");
  return ((SplayNode<T,C>*)i)->cont;
}

template <class T,class C>
inline void SplayMap<T,C>::clear() {
  _kill(root);
  count = 0;
  root = 0;
}

template <class T,class C>
inline int SplayMap<T,C>::contains(T& key) {
  return seek(key) != 0;
}
#include "SplayMap.cc"
#endif
