/* this might be a good standard interface for termio */

extern "C" {
#ifdef m88k
#include <termios.h>
#else
#include <termio.h>
#include <fcntl.h>
#endif
#ifndef VSUSP
#define termios termio
#define tcgetattr(fd,p) ioctl(fd,TCGETA,p)
#define tcsetattr(fd,cmd,p) ioctl(fd,cmd,p)
#define tcdrain(fd)	ioctl(fd,TCSBRK,1)
#define tcsendbreak(fd,dur)	ioctl(fd,TCSBRK,0)
#define TCSAFLUSH 	TCSETAF
#define TCSADRAIN 	TCSETAW
#define TCSANOW		TCSETA
#endif
}

extern "C" int tcgethwhand(int), tcsethwhand(int,int);
extern "C" int lock_tty(const char *);
extern "C" void unlock_tty(const char *);

class Termios {
  struct termios sav, cur;
  int fd;
  int savrc;
public:
  Termios() { fd = -1; }
  Termios(const char *n,int);
  Termios(int f) { fd = f; save(); }
  int open(const char *,int);
  void open(int fd);
  void close();
  operator int() const { return fd; }
  int operator == (int f) { return fd == f; }
  void modem(bool flag = true) {
    if (flag)
      cur.c_cflag &= ~CLOCAL;
    else
      cur.c_cflag |= CLOCAL;
  }
  void brkint(bool flag);
  unsigned baud() const;
  unsigned baud(unsigned);
  int hwhand() const { return tcgethwhand(fd); }
  int hwhand(int i);
  int isatty() const { return savrc == 0; }
  const struct termios &oldtty() const { return sav; }
  void save();
  void echo(bool);
  int raw(char cnt = 1);
  int set() { return tcsetattr(fd,TCSADRAIN,&cur); }
  int setnow() { return tcsetattr(fd,TCSAFLUSH,&cur); }
  struct termios *operator->() { return &cur; }
  int rawmode() const;
  void brk(int dur = 0) { tcsendbreak(fd,dur); }
  void drain() { tcdrain(fd); }
  virtual void error(const char *n);
  virtual ~Termios();
};
