#pragma interface
#include <cipher.h>

/** This "cipher" does a bit permutation.  The bit permutation does not
    add any security.  It is useful only for an official DES implementation.
    The "key" is an array of 64 bytes giving the source bit position 
    of each output bit.
 */
class BitPerm: public BlockCipher {
  unsigned long (*perm)[16][2];
  static void perminit(unsigned long perm[16][16][2],const char p[64]);
  static void permute(unsigned long perm[16][16][2],const char *ib,char *ob);
public:
  BitPerm() { perm = 0; blksize = 8; }
  operator const void *() const { return perm; }
  void setkey(const char *p,int klen);
  void operator=(const char p[64]) { setkey(p,64); }
  void encrypt(const char *ib,char *ob) { permute(perm,ib,ob); }
  void decrypt(const char *ib,char *ob) { permute(perm?perm + 16:0,ib,ob); }
  bool isweak() const { return true; }
  ~BitPerm() { delete [] perm; }
};

class DESBase: public BlockCipher {
protected:
  static BitPerm iperm;
  static int usecnt;
  static long (*sp)[64];		// Combined S and P boxes
  static void spinit(long sp[8][64]);	// init sp
public:
  enum Desmode { STANDARD, FAST, BIGKEY } desmode;
  DESBase(Desmode mode);
  ~DESBase();
};

class DES: public DESBase {
  long f(unsigned long r,unsigned char subkey[8]);

  unsigned char kn[16][8];	// 8 6-bit subkeys for each of 16 rounds
				// initialized by setkey()
public:
  DES(Desmode mode = STANDARD): DESBase(mode) { }
  void setkey(const char *,int = -1);
  void encrypt(const char *ib,char *ob);
  void decrypt(const char *ib,char *ob);
  bool isweak() const;
  ~DES() { }
};

class DES3: public DESBase {
  DES des1;
  DES des2;
public:
  DES3(Desmode);
  void setkey(const char *,int);	// 16 byte key
  void encrypt(const char *ib,char *ob);
  void decrypt(const char *ib,char *ob);
  ~DES3();
};
